@@ -31,19 +31,36 @@ BUFFER_SIZE = 2048
 #
 class DefaultScanner(Scanner):
     ''' scan input stream and iterate UCS-2 code points '''
-    __data = None
+    _data = None
     _ucs4 = False
 
     def __init__(self, ucs4=False):
+        """
+        >>> scanner = DefaultScanner()
+        >>> scanner._ucs4
+        False
+        """
         self._ucs4 = ucs4
 
     def assign(self, value, termenc):
-        self.__data = unicode(value, termenc, 'ignore')
+        """
+        >>> scanner = DefaultScanner()
+        >>> scanner.assign("01234", "ascii")
+        >>> scanner._data
+        u'01234'
+        """
+        self._data = unicode(value, termenc, 'ignore')
 
     def __iter__(self):
+        """
+        >>> scanner = DefaultScanner()
+        >>> scanner.assign("abcde", "UTF-8")
+        >>> print [ c for c in scanner ]
+        [97, 98, 99, 100, 101]
+        """
         if self._ucs4:
             c1 = 0
-            for x in self.__data:
+            for x in self._data:
                 c = ord(x) 
                 if c >= 0xd800 and c <= 0xdbff:
                     c1 = c - 0xd800
@@ -53,7 +70,7 @@ class DefaultScanner(Scanner):
                     c1 = 0
                 yield c
         else:
-            for x in self.__data:
+            for x in self._data:
                 yield ord(x)
 
 ################################################################################
@@ -62,10 +79,26 @@ class DefaultScanner(Scanner):
 #
 class SimpleParser(Parser):
     ''' simple parser, don't parse ESC/CSI/string seqneces '''
-    def __init__(self):
-        pass
+
+    class _MockContext:
+
+        output = []
+
+        def __iter__(self):
+            for i in [1, 2, 3, 4, 5]:
+                yield i 
+
+        def dispatch_char(self, c):
+            self.output.append(c)
 
     def parse(self, context):
+        """
+        >>> parser = SimpleParser()
+        >>> context = SimpleParser._MockContext()
+        >>> parser.parse(context)
+        >>> context.output
+        [1, 2, 3, 4, 5]
+        """
         for c in context:
             context.dispatch_char(c)
  
@@ -81,6 +114,20 @@ _STATE_CSI_INTERMEDIATE = 4
 _STATE_OSC = 6
 _STATE_STR = 7 
 
+class _MockHandler:
+
+    def handle_csi(self, context, parameter, intermediate, final):
+        print (parameter, intermediate, final)
+
+    def handle_esc(self, context, intermediate, final):
+        print (intermediate, final)
+
+    def handle_control_string(self, context, prefix, value):
+        print (prefix, value)
+
+    def handle_char(self, context, c):
+        print (c)
+
 class DefaultParser(Parser):
     ''' parse ESC/CSI/string seqneces '''
 
@@ -523,7 +570,8 @@ class DefaultPTY(PTY):
         os.write(self._master, data)
 
     def flush(self):
-        os.fsync(self._master)
+        pass
+        #os.fsync(self._master)
 
     def xoff(self):
         #fcntl.ioctl(self._master, termios.TIOCSTOP, 0)
@@ -640,7 +688,12 @@ class Session:
             inputhandler.handle_end(inputcontext)
             outputhandler.handle_end(outputcontext)
 
+
+def _test():
+    import doctest
+    doctest.testmod()
+
 ''' main '''
 if __name__ == '__main__':    
-    pass
+    _test()
 
